/**
 * This file was automatically generated by openapi-codegeneration.
 * DO NOT MODIFY IT BY HAND. Instead, modify the source OpenAPI file,
 * and run openapi-codegeneration to regenerate this file.
 */
import * as Signatures from './signatures.js';
import * as Types from './types.js';
/**
 * This error class should be used if an error occurs during validation of a request/response.
 * @category Errors
 */
export declare class ValidationError extends Error {
    errors: unknown;
    constructor(message: string, errors: unknown);
}
/**
 * This error class should be used if an error occurs while fetching a request.
 * @category Errors
 */
export declare class FetchError extends Error {
    constructor(message: string);
}
/**
 * This error class should be used if the response of the server has a status
 * greater than or equal to 400. This error should contain the validated response.
 * @category Errors
 */
export declare class UnsuccessfulRequestError extends Error {
    response: Types.ResponseData;
    constructor(message: string, response: Types.ResponseData);
}
/**
 * This error class should be used if the validation of an url fails.
 * @category Errors
 */
export declare class InvalidUrlError extends Error {
    constructor(message: string);
}
/**
 * Client for the use of the Crosslab API
 * @category Client
 */
export declare class Client {
    readonly baseUrl: string;
    readonly serviceUrl: string;
    accessToken: string;
    private fixedHeaders;
    private fetch;
    /**
     * @category Constructors
     */
    constructor(baseUrl: string, options: {
        serviceUrl?: string;
        accessToken?: string;
        fixedHeaders?: [string, string][];
    });
    /**
     * Books an experiment.
     *
     * @param body
     * @param options.url
     * Url of the  to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The booking was accepted. The booking status will display whether the booking of all devices was successful.
     */
    newBooking(body: {
        /**
         * List of devices which should be added.
         */
        Devices: Types.Device<'request'>[];
        /**
         * A time slot represents a slice of time used for bookings.
         */
        Time: Types.Timeslot<'request'>;
        /**
         * Type of booking. Currently, only one type is defined, but others might follow (e.g. priority booking). If empty, 'normal' is assumed.
         */
        Type?: 'normal';
        [k: string]: unknown;
    }, options?: {
        headers?: [string, string][];
        url?: string;
    }): Promise<Signatures.NewBookingSuccessResponse['body']>;
    /**
       * Allows the addition of devices to a booking (removing of devices is not supportet) or the registration of callbacks. For adding devices, the scope 'booking:use' is required.
   
       *
       * @param url
       * Url of the resource to be accessed.
       * @param body
       *
       * @throws {@link FetchError | FetchError }
       * Thrown if fetch fails.
       * @throws {@link ValidationError | ValidationError }
       * Thrown if the request/response validation fails.
       * @throws {@link InvalidUrlError | InvalidUrlError }
       * Thrown if the provided url is not valid for this request.
       * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
       * Thrown if response is validated but has status greater than or equal to 400.
       *
       * @returns
       * The booking change was accepted. The booking status will display whether the booking of all devices was successful.
       */
    updateBooking(url: string, body: {
        /**
         * Expresses whether the devices should be locked. Must match current status of booking. Is assumed to be false if not set.
         */
        Locked?: boolean;
        /**
         * List of devices which should be added.
         */
        Devices?: Types.Device<'request'>[];
        [k: string]: unknown;
    } | {
        /**
         * Callback which should be called at changes.
         */
        Callback?: string;
        [k: string]: unknown;
    }, options?: {
        headers?: [string, string][];
    }): Promise<Signatures.UpdateBookingSuccessResponse['body']>;
    /**
     * Cancels a booking, as long as the booking was originally done by you.
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The booking was cancelled. All associated devices were released.
     */
    deleteBooking(url: string, options?: {
        headers?: [string, string][];
    }): Promise<void>;
    /**
     * Returns whether a list of devices is currently booked for a user.
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Returns the current booking.
     */
    getBooking(url: string, options?: {
        headers?: [string, string][];
    }): Promise<Signatures.GetBookingSuccessResponse['body']>;
    /**
     * Allows selected persons (like lab manager) to remove a user booking. To avoid mistakes, this is a different path than normal delete.
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The booking was cancelled. All associated devices were released.
     */
    destroyBooking(url: string, options?: {
        headers?: [string, string][];
    }): Promise<void>;
}
//# sourceMappingURL=client.d.ts.map